package background;

import infrastructure.DeterministicNodeIdFactory;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.util.LinkedList;

import application.Cuckoo;
import application.Profile;

import rice.environment.Environment;
import rice.pastry.PastryNode;
import rice.pastry.PastryNodeFactory;
import rice.pastry.socket.SocketPastryNodeFactory;
import winterwell.jtwitter.Twitter;


public class ScriptRunBKG {
	public static final String TWEETS = "./tweets/";
	public static final String BKG_CFG = "./bkg_cfg";
	public static final int INTERVAL = 60000;
	
	private LinkedList<Cuckoo> cuckoos = new LinkedList<Cuckoo>();
	
	private DeterministicNodeIdFactory nidFactory = null;
	private PastryNodeFactory factory;
	private Environment env;
	private int bootport;
	private InetSocketAddress bootaddress;
	
	public ScriptRunBKG() {
		// Loads pastry settings
		env = new Environment();
		env.getParameters().setString("nat_search_policy", "never");
		
		bootport = 9001;
		InetAddress bootaddr;
		
		try {
			bootaddr = InetAddress.getLocalHost();
			bootaddress = new InetSocketAddress(bootaddr, bootport);
		} catch(IOException ioe) {
			ioe.printStackTrace();
		}
		
		initCuckoos();
		//run();
	}
	
	public void initCuckoos() {
		try {
			BufferedReader is = new BufferedReader(new FileReader(BKG_CFG));
			String cfgline;
			while((cfgline = is.readLine()) != null) {
				cfgline = cfgline.trim();
				if(cfgline.compareTo("") != 0 && cfgline.substring(0, 2).compareTo("//") != 0) {
					String[] split  = cfgline.split("#");
					String userid   = split[0];
					String username = split[1];
					String password = split[2];
					int bindport    = Integer.valueOf(split[3]).intValue();
					/////////////Only first use!!///////////////////
					Twitter twitter = new Twitter(username, password);
					//
					Profile profile = new Profile(twitter.show(Long.valueOf(userid).longValue()));
					System.out.print(profile.toString());
					//
					init1cuckoo(userid, username, password, profile, bindport);
					////////////////////////////////////////////////
			
					//init1cuckoo(userid, username, password, null, bindport);
				}
			}
			is.close();
		} catch(IOException ioe) {
			ioe.printStackTrace();
		}
	}
	
	private void init1cuckoo(String userid, String um, String pwd, Profile profile, int bindport) {
		nidFactory = new DeterministicNodeIdFactory(userid);
		//if(profile != null) {
		//	accountDB.addAccount(new Account(um, pwd, userid));
		//}
		try {
			factory = new SocketPastryNodeFactory(nidFactory, bindport, env);
			PastryNode node = factory.newNode(nidFactory.generateNodeId());
			node.boot(bootaddress);
			
			//the node may require sending several messages to fully boot into the ring
			synchronized (node) {
				while (!node.isReady() && !node.joinFailed()) {
					// delay so we don't busy-wait
					node.wait(500);
					// abort if can't join
					if (node.joinFailed()) {
						System.out.println("Could not join the FreePastry ring.  Reason:" + node.joinFailedReason());
					}
				}
			}
			System.out.println("Finished creating new node " + node + um);
			
			Twitter twitter = new Twitter(um, pwd);
			twitter.setCount(5);
			Cuckoo cuckoo = new Cuckoo(factory, node, userid, null, twitter);
			cuckoo.addMyProfile(profile);
			
			cuckoos.add(cuckoo);
			
		} catch(IOException ioe) {
			ioe.printStackTrace();
		} catch (InterruptedException ie) {
			ie.printStackTrace();
		}
	}
	
	
	//Use different threads to publish contents
	public void run() {
		System.out.println(cuckoos.size());
	
		for(int i=0; i<cuckoos.size(); i++) {
			cuckoos.get(i).follow("113601199");
		}
		
		//cuckoos.get(2).follow("148658740");
		//cuckoos.get(2).follow("10434312");
		
		for(int i=0; i<cuckoos.size(); i++) {
			Cuckoo tmpCuc = cuckoos.get(i);
			//Add!!!
			Tweet tweet = new Tweet(tmpCuc);
			System.out.println("-------------" + tmpCuc.getUserId());
			tweet.start();
		}
	}
	
	private class Tweet extends Thread {
		public Cuckoo cuckoo;
		
		public Tweet(Cuckoo cuc) {
			cuckoo = cuc;
		}
		
		public void run() {
			try {
				BufferedReader cdis = new BufferedReader(new FileReader(TWEETS+cuckoo.getUserId()));
				String tweetLine;
				while((tweetLine = cdis.readLine()) != null) {
					tweetLine = tweetLine.trim();
					//System.out.println(tweetLine);
					cuckoo.writeStatus(tweetLine);
					long wtime = (long) (Math.random() * INTERVAL);
					sleep(wtime);
				}
			} catch(IOException ioe) {
				ioe.printStackTrace();
			} catch (InterruptedException ie) {
				ie.printStackTrace();
			}
		}
	}
	/*
	public void run() throws Exception {

		LoadSample ls = new LoadSample();
		ls.loadTestInfo();
		//construct 
		ArrayList<String> al = ls.getKeys();
		Iterator<String> iterator = al.iterator();
		while(iterator.hasNext()) {
			String uid = iterator.next();
			Init1Cuckoo(uid, ls.getProfile(uid));
		}
		
		iterator = al.iterator();
		while(iterator.hasNext()) {
			String uid = iterator.next();
			ArrayList<String> fl = ls.getFollowees(uid);
			if(fl != null) {
				Iterator<String> iter = fl.iterator();
				while(iter.hasNext()) {
					cuckoos.get(uid).follow(iter.next());
				}
			}
		}
		
		iterator = al.iterator();
		while(iterator.hasNext()) {
			String uid = iterator.next();
			ArrayList<String> tl = ls.getTweets(uid);
			if(tl != null) {
				Iterator<String> iter = tl.iterator();
				while(iter.hasNext()) {
					cuckoos.get(uid).writeStatus(iter.next());
				}
			}
		}
	}
	*/
	public static void main(String[] args) {
		ScriptRunBKG scriptRunBackground = new ScriptRunBKG();
		scriptRunBackground.run();
	}
	
	
}
