package database;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import toolbox.Partner;
import toolbox.PartnerMap;
import toolbox.PartnerMap.PartnerQueue;

public class PartnerXMLDB {
	private String path;
	private String rootName;
	
	public PartnerXMLDB(String dir, String path, String root) {
		File file = new File(path);
		this.path = path;
		this.rootName = root;
		
		try {
			if(file.exists() == false) {
				File fDir = new File(dir);
				if(fDir.exists() == false) {
					fDir.mkdir();
				}
				file.createNewFile();
				createXmlDoc();
			}			
			
		} catch(FileNotFoundException e1) {
			System.out.println("FileNotFoundException @ createXmlDoc in PartnerXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ createXmlDoc in PartnerXMLDB");
		}
	}
	
	private void createXmlDoc() throws FileNotFoundException, IOException {
		Element root = new Element(rootName);
		Document Doc = new Document(root);
		
		XMLOutputter XMLOut = new XMLOutputter();
		XMLOut.output(Doc, new FileOutputStream(path));
	}
	
	public synchronized void initPartnerList(PartnerMap partnerMap) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			
			for (Element e : list) {
				String fid = e.getAttributeValue("followeeId");
				String ids = e.getChildText("partnerIdList");
				String healths = e.getChildText("healthList");
				String sims    = e.getChildText("simList");
				
				String[] isplit = ids.split("#");
				String[] hsplit = healths.split("#");
				String[] ssplit = sims.split("#");
				for(int i=0; i<isplit.length; i++) {
					partnerMap.initAdd(fid, new Partner(fid, isplit[i], "", Integer.valueOf(hsplit[i]).intValue(), Integer.valueOf(ssplit[i]).intValue()));
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ initPartnerList in PartnerXMLDB");
		} catch(FileNotFoundException e2) {
			System.out.println("FileNotFoundException @ initPartnerList in PartnerXMLDB");
		} catch(IOException e3) {
			System.out.println("IOException @ initPartnerList in PartnerXMLDB");
		}
	}
	
	public synchronized void xmlParsePrint() {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				System.out.println("partner--"+e.getAttributeValue("followeeId")+" "+e.getChildText("partnerIdList"));
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException in xmlParsePrint!");
		} catch(IOException e2) {
			System.out.println("IOException in xmlParsePrint!");
		}
	}
	
	public synchronized void writeBack(HashMap<String, PartnerQueue> partnerMap) {
		try {
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(path);

			Element root = doc.getRootElement();
			List<Element> list = root.getChildren();
			
			//First update the out-of-date elements in the XML-based DB
			String id;
			PartnerQueue pq;
			PartnerRecordBlock prBlock;
			for (Element e : list) {
				id = e.getAttributeValue("followeeId");
				pq = partnerMap.remove(id);
				if(pq != null && pq.getLength() != 0) {
					prBlock = new PartnerRecordBlock(pq);
					e.getChild("partnerIdList").setText(prBlock.getPartnerIdList());
					e.getChild("healthList").setText(prBlock.getHealthList());
					e.getChild("simList").setText(prBlock.getSimList());
				}
			}
		
			if(partnerMap.size() != 0) {
				Set<String> keyset = partnerMap.keySet();
				Iterator<String> iterator = keyset.iterator();
			
				while(iterator.hasNext()) {
					id = iterator.next();
					pq = partnerMap.get(id);
					prBlock = new PartnerRecordBlock(pq);
				
					Element element = new Element("followeeId");
					element.setAttribute("followeeId", prBlock.getFolloweeId());
					Element element1 = new Element("partnerIdList");
					element1.setText(prBlock.getPartnerIdList());
					Element element2 = new Element("healthList");
					element2.setText(prBlock.getHealthList());
					Element element3 = new Element("simList");
					element3.setText(prBlock.getSimList());
				
					element.addContent(element1);
					element.addContent(element2);
					element.addContent(element3);
					root.addContent(element);
				}
			}
			
			doc.setRootElement(root);
			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
			
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ writeBack in PartnerXMLDB");
		} catch(FileNotFoundException e2) {
			System.out.println("FileNotFoundException @ writeBack in PartnerXMLDB");
		} catch(IOException e3) {
			System.out.println("IOException @ writeBack in PartnerXMLDB");
		}
	}
	
	//Record Block, used to write back to the XML-based DB
	public class PartnerRecordBlock {
		private String followeeId;
		private String partnerIdList;  //user # as the delimiter
		private String healthList;     //user # as the delimiter
		private String simList;        //user # as the delimiter
		
		public PartnerRecordBlock(String fid, String list) {
			followeeId = fid;
			partnerIdList = list;
		}
		
		public PartnerRecordBlock(PartnerQueue pqueue) {
			followeeId = pqueue.getFolloweeId();
			partnerIdList = pqueue.partners2string();
			healthList = pqueue.healthes2string();
			simList = pqueue.sims2string();
			//pqueue.buildLists(partnerIdList, healthList, simList);
			//System.out.println("partnerIdList ---> " + partnerIdList);
		}
		
		public String getFolloweeId() {
			return followeeId;
		}
		
		public String getPartnerIdList() {
			return partnerIdList;
		}
		
		public String getHealthList() {
			return healthList;
		}
		
		public String getSimList() {
			return simList;
		}
	}
	
	//just for test
	public static void main(String[] args) {
		PartnerXMLDB xmlDB = new PartnerXMLDB("xml", "xml/partners.xml", "Partners");
		PartnerMap pl = new PartnerMap(xmlDB);
		
		pl.printPartnerMap(1);
		System.out.println("----------------------------");
		pl.printPartnerMap(2);
		System.out.println("----------------------------");
		
		pl.runtimeAdd("123", new Partner("123", "1", "", 5, 1));
		pl.runtimeAdd("123", new Partner("123", "2", "", 5, 1));
		pl.runtimeAdd("123", new Partner("123", "3", "", 5, 1));
		pl.runtimeAdd("123", new Partner("123", "4", "", 5, 1));
		pl.runtimeAdd("123", new Partner("123", "5", "", 5, 1));
		pl.runtimeAdd("123", new Partner("123", "6", "", 5, 1));
		pl.runtimeAdd("123", new Partner("123", "7", "", 5, 1));
		pl.runtimeAdd("234", new Partner("234", "8", "", 5, 1));
		pl.runtimeAdd("234", new Partner("234", "9", "", 5, 1));
		pl.runtimeAdd("345", new Partner("345", "10", "", 5, 1));
		pl.runtimeAdd("234", new Partner("234", "19", "", 5, 1));
		
		pl.runtimeAdd("456", new Partner("456", "19", "", 5, 1));
		
		pl.printPartnerMap(1);
		System.out.println("----------------------------");
		pl.printPartnerMap(2);
		System.out.println("----------------------------");
		
		
		//pl.printPartnerMap(1);
		pl.writeBack();
		
		xmlDB.xmlParsePrint();
	}
}
