package database;

import infrastructure.DeterministicNodeIdFactory;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import application.AppPar;
import application.Profile;

import rice.p2p.commonapi.Id;
import rice.p2p.util.BloomFilter;
import toolbox.HealthIndex;
import toolbox.UTPair;
import winterwell.jtwitter.Twitter;


public class ProfileXMLDB {
	private String path;
	private String rootName;

	public ProfileXMLDB(String dir, String path, String root) {
		File file = new File(path);
		this.path = path;
		this.rootName = root;
		
		try {
			if(file.exists() == false) {
				File fDir = new File(dir);
				if(fDir.exists() == false) {
					fDir.mkdir();
				}
				file.createNewFile();
				createXmlDoc();
			}			
			
		} catch(FileNotFoundException e1) {
			System.out.println("FileNotFoundException @ createXmlDoc in ProfileXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ createXmlDoc in ProfileXMLDB");
		}
	}
	
	private void createXmlDoc() throws FileNotFoundException, IOException {
		Element root = new Element(rootName);
		Document Doc = new Document(root);

		// Element elements = new Element("status");
		// elements.setAttribute("ID", new Integer(0).toString());
		// elements.addContent(new Element("text").setText("Welcome!"));
		// elements.addContent(new Element("username").setText("name" + i));
		// elements.addContent(new Element("password").setText("@" + i+
		// ".com"));
		// root.addContent(elements);

		XMLOutputter XMLOut = new XMLOutputter();
		XMLOut.output(Doc, new FileOutputStream(path));
	}

	public String getNameFromId(String userId) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				if (e.getAttributeValue("userId").compareTo(userId) == 0) {
					return e.getChildText("username");
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException in xmlParsePrint!");
		} catch(IOException e2) {
			System.out.println("IOException in xmlParsePrint!");
		}
		return null;
	}
	
	public synchronized void xmlParsePrint() {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				//System.out.println("User ID:  " + e.getAttributeValue("userId"));
				//System.out.println("Node ID:  " + e.getChildText("nodeId"));
				//System.out.println("Username: " + e.getChildText("username"));
				//System.out.println("Location: " + e.getChildText("location"));
				//System.out.println("Web:      " + e.getChildText("web"));
				//System.out.println("Bio:      " + e.getChildText("bio"));
				//System.out.println("Last modified time: " + e.getChildText("time"));
				System.out.println("xml"+e.getAttributeValue("userId")+" "+e.getChildText("username")+" "+e.getChildText("location")+" "+e.getChildText("web")+" "+e.getChildText("bio")+" "+e.getChildText("time")+" "+e.getChildText("health"));
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException in xmlParsePrint!");
		} catch(IOException e2) {
			System.out.println("IOException in xmlParsePrint!");
		}
	}

	public synchronized LinkedList<Id> getAllNodelIds() {
		LinkedList<Id> idList = new LinkedList<Id>();
		DeterministicNodeIdFactory nidFactory;

		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				nidFactory = new DeterministicNodeIdFactory(e.getAttributeValue("userId"));
				idList.add(nidFactory.generateNodeId());
			}

		} catch (JDOMException e1) {
			System.out.println("JDOMException in getAllIds!");
		} catch (IOException e2) {
			System.out.println("IOException in getAllIds!");
		}
		
		return idList;
	}
	
	public synchronized LinkedList<String> getAllUserIds() {
		LinkedList<String> uIdList = new LinkedList<String>();

		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				uIdList.add(e.getAttributeValue("userId"));
			}

		} catch (JDOMException e1) {
			System.out.println("JDOMException in getAllIds!");
		} catch (IOException e2) {
			System.out.println("IOException in getAllIds!");
		}
		
		return uIdList;
	}
	
	public synchronized LinkedList<UTPair> getAllUTPair() {
		LinkedList<UTPair> pairList = new LinkedList<UTPair>();
		
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				pairList.addLast(new UTPair(e.getAttributeValue("userId"), e.getChildText("time")));
			}

		} catch (JDOMException e1) {
			System.out.println("JDOMException in getAllIds!");
		} catch (IOException e2) {
			System.out.println("IOException in getAllIds!");
		}
		
		return pairList;
	}
	
	public synchronized UTPair getUTPair(String uid) {
		UTPair utpair = null;
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				if(e.getAttributeValue("userId").compareTo(uid) == 0) {
					return new UTPair(e.getAttributeValue("userId"), e.getChildText("time"));
				}
			}
		} catch (JDOMException e1) {
			System.out.println("JDOMException in getAllIds!");
		} catch (IOException e2) {
			System.out.println("IOException in getAllIds!");
		}
		return null;
	}
	
	public synchronized LinkedList<Profile> getAllProfile() {
		LinkedList<Profile> profileList = new LinkedList<Profile>();
		
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				profileList.addLast(new Profile(e.getAttributeValue("userId"), e.getChildText("username"), e.getChildText("location"), e.getChildText("web"), e.getChildText("bio")));
			}
		} catch (JDOMException e1) {
			System.out.println("JDOMException in getAllProfile!");
		} catch (IOException e2) {
			System.out.println("IOException in getAllProfile!");
		}
		
		return profileList;
	}

	public synchronized void addProfile(Twitter.User userProfile) {
		addProfile(new Profile(userProfile));
	}
	
	public synchronized void addProfile(Profile profile) {
		try {
			if(exist(profile.getUserId()) == true) {
				return;
			}
			
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(path);

			Element root = doc.getRootElement();
			
			Element element0 = new Element("userId");
			element0.setAttribute("userId", profile.getUserId());
			
			Element element2 = new Element("username");
			element2.setText(profile.getUserName());

			Element element3 = new Element("location");
			element3.setText(profile.getLocation());

			Element element4 = new Element("web");
			element4.setText(profile.getWeb());

			Element element5 = new Element("bio");
			element5.setText(profile.getBio());

			Element element6 = new Element("time");
			element6.setText("0");
			
			Element element7 = new Element("health");
			element7.setText(String.valueOf(HealthIndex.INIT_BLOOD));
			
			element0.addContent(element2);
			element0.addContent(element3);
			element0.addContent(element4);
			element0.addContent(element5);
			element0.addContent(element6);
			element0.addContent(element7);

			root.addContent(element0);
			doc.setRootElement(root);

			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
		
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ addProfile in ProfileXMLDB");
		} catch(FileNotFoundException e2) {
			System.out.println("FileNotFoundException @ addProfile in ProfileXMLDB");
		} catch(IOException e3) {
			System.out.println("IOException @ addProfile in ProfileXMLDB");
		}
	}

	public synchronized void deleteProfile(String userId) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);

			Element root = doc.getRootElement();

			List<Element> list = root.getChildren();

			for (Element e : list) {
				if (e.getAttributeValue("userId").compareTo(userId) == 0) {
					root.removeContent(e);
					break;// ?? What should we do?
				}
			}

			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
			//numOfProfile -= 1;
		} catch(JDOMException jdome) {
			System.out.println("JDOMException in deleteProfile @ ProfileXMLDB"); 
		} catch(IOException ioe) {
			System.out.println("IOException in deleteProfile @ ProfileXMLDB"); 
		}
	}

	public synchronized void updateProfile(String userId, Profile profile) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);

			Element root = doc.getRootElement();

			List<Element> list = root.getChildren();
			for (Element e : list) {
				if (e.getAttributeValue("userId").compareTo(userId) == 0) {
					e.getChild("username").setText(profile.getUserName());
					e.getChild("location").setText(profile.getLocation());
					e.getChild("web").setText(profile.getWeb());
					e.getChild("bio").setText(profile.getBio());
				}
			}

			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ updateProfile in ProfileXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ updateProfile in ProfileXMLDB");
		}
	}

	public synchronized void updateUserName(String userId, String username)
			throws JDOMException, IOException {
		SAXBuilder builder = new SAXBuilder();
		InputStream file = new FileInputStream(path);
		Document doc = builder.build(file);

		Element root = doc.getRootElement();

		List<Element> list = root.getChildren();
		for (Element e : list) {
			if (e.getAttributeValue("userId").compareTo(userId) == 0) {
				e.getChild("username").setText(username);
			}
		}

		XMLOutputter out = new XMLOutputter();
		out.output(doc, new FileOutputStream(path));
	}

	public synchronized void updateLocation(String userId, String location)
			throws JDOMException, IOException {
		SAXBuilder builder = new SAXBuilder();
		InputStream file = new FileInputStream(path);
		Document doc = builder.build(file);

		Element root = doc.getRootElement();

		List<Element> list = root.getChildren();
		for (Element e : list) {
			if (e.getAttributeValue("userId").compareTo(userId) == 0) {
				e.getChild("location").setText(location);
			}
		}

		XMLOutputter out = new XMLOutputter();
		out.output(doc, new FileOutputStream(path));
	}

	public synchronized void updateWeb(String userId, String web) 
			throws JDOMException, IOException {
		SAXBuilder builder = new SAXBuilder();
		InputStream file = new FileInputStream(path);
		Document doc = builder.build(file);

		Element root = doc.getRootElement();

		List<Element> list = root.getChildren();
		for (Element e : list) {
			if (e.getAttributeValue("userId").compareTo(userId) == 0) {
				e.getChild("web").setText(web);
			}
		}

		XMLOutputter out = new XMLOutputter();
		out.output(doc, new FileOutputStream(path));
	}
	
	public synchronized void updateTime(Twitter.Status st) {
		/*try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);
			
			Element root = doc.getRootElement();
			
			List<Element> list = root.getChildren();
			for (Element e : list) {
				if (e.getAttributeValue("userId").compareTo(String.valueOf(st.getUser().id)) == 0) {
					e.getChild("time").setText(String.valueOf(st.createdAt.getTime()));
				}
			}
			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
			
		} catch(JDOMException jdome) {
			System.out.println("JDOMException in updateTime() @ ProfileXMLDB");
		} catch(IOException ioe) {
			System.out.println("IOException in updateTime() @ ProfileXMLDB");
		}*/
		updateTime(String.valueOf(st.getUser().getId()), String.valueOf(st.getCreatedAt().getTime()));
	}
	
	public synchronized void updateTime(String userId, String time) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);

			Element root = doc.getRootElement();

			List<Element> list = root.getChildren();
			for (Element e : list) {
				if (e.getAttributeValue("userId").compareTo(userId) == 0) {
					e.getChild("time").setText(time);
				}
			}

			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
			
		} catch(JDOMException jdome) {
			System.out.println("JDOMException in updateTime() @ ProfileXMLDB");
		} catch(IOException ioe) {
			System.out.println("IOException in updateTime() @ ProfileXMLDB");
		}
	}
	
	public synchronized void updateTime(String userId, long time) throws JDOMException, IOException {
		updateTime(userId, String.valueOf(time));
	}

	public synchronized void updateBio(String userId, String bio) throws JDOMException, IOException {
		SAXBuilder builder = new SAXBuilder();
		InputStream file = new FileInputStream(path);
		Document doc = builder.build(file);

		Element root = doc.getRootElement();

		List<Element> list = root.getChildren();
		for (Element e : list) {
			if (e.getAttributeValue("userId").compareTo(userId) == 0) {
				e.getChild("bio").setText(bio);
			}
		}

		XMLOutputter out = new XMLOutputter();
		out.output(doc, new FileOutputStream(path));
	}
	
//	public int getNumOfProfile() {
//		return numOfProfile;
//	}
	
	public synchronized Profile getProfile(String userId) {
		try {
			SAXBuilder builder = new SAXBuilder();
		
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);
		
			Element root = doc.getRootElement();
		
			List<Element> list = root.getChildren();
			for (Element e : list) {
				if ( e.getAttributeValue("userId").compareTo(userId) == 0 ) {
					return new Profile(e.getAttributeValue("userId"), e.getChildText("username"), 
										e.getChildText("location"), e.getChildText("web"), e.getChildText("bio"));
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ getProfile in ProfileXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ getProfile in ProfileXMLDB");
		}
		return null;
	}
	
	public synchronized long getLastTime(String userid) {
		long time = 0;
		try {
			SAXBuilder builder = new SAXBuilder();
			
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);
		
			Element root = doc.getRootElement();
		
			List<Element> list = root.getChildren();
			for (Element e : list) {
				if ( e.getAttributeValue("userId").compareTo(userid) == 0 ) {
					String t = e.getChildText("time");
					time = Long.valueOf(t).longValue();
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ getProfile in ProfileXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ getProfile in ProfileXMLDB");
		}
		return time;
	}

	//Whether a user exists in the XML file
	public synchronized boolean exist(String userId) {
		try {
			SAXBuilder builder = new SAXBuilder();
		
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);
		
			Element root = doc.getRootElement();
		
			List<Element> list = root.getChildren();
			for (Element e : list) {
				if ( e.getAttributeValue("userId").compareTo(userId) == 0 ) {
					return true;
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ getProfile in ProfileXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ getProfile in ProfileXMLDB");
		}
		return false;
	}
	
	//Init the health index
	//healthIndice is the input parameter
	public synchronized void initHealthIndex(HashMap<String, Integer> healthIndice) {
		try {
			SAXBuilder builder = new SAXBuilder();
		
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);
		
			Element root = doc.getRootElement();
		
			List<Element> list = root.getChildren();
			for (Element e : list) {
				if(Integer.valueOf(e.getChildText("health")) > 0) {
					healthIndice.put(e.getAttributeValue("userId"), Integer.valueOf(e.getChildText("health")));
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ initHealthIndex in ProfileXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ initHealthIndex in ProfileXMLDB");
		}
	}
	
	public synchronized void refreshHealthIndex(HashMap<String, Integer> healthIndice) {
		try {
			SAXBuilder builder = new SAXBuilder();
		
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);
		
			Element root = doc.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				String uid = e.getAttributeValue("userId");
				Integer l = healthIndice.get(uid);
				//System.out.println(l);
				if(l != null) {
					e.getChild("health").setText(String.valueOf(l));
					if( l.compareTo(new Integer(0)) == 0) {
						//filter the dead users in the healthIndice
						healthIndice.remove(uid);
					}
				}
			}
			//write to file
			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
			
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ refreshHealthIndex in ProfileXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ refreshHealthIndex in ProfileXMLDB");
		}
	}
	
	public void addNewAttribute(String attrName, int i) {
		addNewAttribute(attrName, String.valueOf(i));
	}
	
	public void addNewAttribute(String attrName, String defaultValue) {
		try {
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(path);

			Element root = doc.getRootElement();
			
			List<Element> list = root.getChildren();
			for (Element e : list) {
				Element newElement = new Element(attrName); 
				newElement.setText(defaultValue);
				e.addContent(newElement);
			}

			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
		
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ addProfile in ProfileXMLDB");
		} catch(FileNotFoundException e2) {
			System.out.println("FileNotFoundException @ addProfile in ProfileXMLDB");
		} catch(IOException e3) {
			System.out.println("IOException @ addProfile in ProfileXMLDB");
		}
	}
	
	public BloomFilter buildBloomFilter() {
		BloomFilter bloomfilter = new BloomFilter(AppPar.BLOOMFILTER_NUM, AppPar.BLOOMFILTER_LEN);
		
		try {
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(path);

			Element root = doc.getRootElement();
		
			List<Element> list = root.getChildren();
			for (Element e : list) {
				String uid = e.getAttributeValue("userId"); 
				bloomfilter.add(uid.getBytes());
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ buildBloomFilter in ProfileXMLDB");
		} catch(FileNotFoundException e2) {
			System.out.println("FileNotFoundException @ buildBloomFilter in ProfileXMLDB");
		} catch(IOException e3) {
			System.out.println("IOException @ buildBloomFilter in ProfileXMLDB");
		}
		
		return bloomfilter;
	}
	
	public synchronized int getNumofEntries() {
		int size = 0;
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			if(list == null) {
				size = 0;
			} else {
				size = list.size();
			}
			
		} catch(JDOMException e1) {
			System.out.println("JDOMException in xmlParsePrint!");
		} catch(IOException e2) {
			System.out.println("IOException in xmlParsePrint!");
		}
		return size;
	}
	
	static public void main(String ars[]) throws JDOMException, IOException {
		ProfileXMLDB xmlDB = new ProfileXMLDB("xml", "xml/profile.xml", "Profile");
		
		xmlDB.addProfile(new Profile("456", "Tianyin", "Goettingen",
				"http://cs.nju.edu.cn/dislab/xuty/xuty.html", "Hello!"));
		//xmlDB.xmlParsePrint();
		//xmlDB.updateProfile("456", new Profile("456", "234", "Tianyin", "Nanjing",
		//		"http://cs.nju.edu.cn/dislab/xuty/xuty.html", "Hello!"));
		//xmlDB.xmlParsePrint();
		
		//xmlDB.updateBio("456", "I'm enjoying the coding!!!");
		//xmlDB.updateTime("456", 12345);
		
		xmlDB.xmlParsePrint();
		
		//System.out.println(xmlDB.getAllUTPair().size());
		
		UTPair.printUTPairList(xmlDB.getAllUTPair());
		
		//Profile p = xmlDB.getProfile("123");
		//p.print();
		
		//System.out.println(xmlDB.getNumFromId("456"));
		//xmlDB.getProfile("456").print();
		
	}

}
