package database;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import toolbox.TrafficLog;

public class TrafficAnalysisXMLDB {
	private String path;
	private String rootName;
	
	public TrafficAnalysisXMLDB(String dir, String path, String root) {
		File file = new File(path);
		this.path = path;
		this.rootName = root;
		
		try {
			if(file.exists() == false) {
				File fDir = new File(dir);
				if(fDir.exists() == false) {
					fDir.mkdir();
				}
				file.createNewFile();
				createXmlDoc();
			}			
			
		} catch(FileNotFoundException e1) {
			System.out.println("FileNotFoundException @ constructor in TrafficAnalysisXMLDB");
		} catch(IOException e2) {
			System.out.println("IOException @ constructor in TrafficAnalysisXMLDB");
		}
	}
	
	private void createXmlDoc() throws FileNotFoundException, IOException {
		Element root = new Element(rootName);
		Document Doc = new Document(root);

		XMLOutputter XMLOut = new XMLOutputter();
		XMLOut.output(Doc, new FileOutputStream(path));
	}
	
	public synchronized void addTrafficLog(TrafficLog tl) {
		boolean t = false;
		while(t == false) {
			try {
				SAXBuilder builder = new SAXBuilder();
				Document doc = builder.build(path);

				Element root = doc.getRootElement();
			
				Element element0 = new Element("time");
				element0.setAttribute("time", new Long(tl.getTime()).toString());

				Element element1 = new Element("traffic");
				element1.setText(String.valueOf(tl.getTraffic()));
			
				Element element2 = new Element("mark");
				element2.setText(tl.getMark());

				Element element3 = new Element("isRead");
				element3.setText(tl.getInOut());
			
				Element element4 = new Element("fromid");
				element4.setText(tl.getFromId());
			
				Element element5 = new Element("toid");
				element5.setText(tl.getToId());
				
				Element element6 = new Element("redundancy");
				element6.setText(tl.getRedundancy());
			
				element0.addContent(element1);
				element0.addContent(element2);
				element0.addContent(element3);
				element0.addContent(element4);
				element0.addContent(element5);
				element0.addContent(element6);

				//root.addContent(0, element0);
				root.addContent(findIndex(root, Long.valueOf(tl.getTime())), element0);
				doc.setRootElement(root);
			
				XMLOutputter out = new XMLOutputter();
				out.output(doc, new FileOutputStream(path));
				t = true;

			} catch(JDOMException e1) {
				System.out.println("JDOMException @ addTrafficLog in TrafficAnalysisXMLDB");
				e1.printStackTrace();
			} catch(FileNotFoundException e2) {
				System.out.println("FileNotFoundException @ addTrafficLog in TrafficAnalysisXMLDB");
			} catch(IOException e3) {
				System.out.println("IOException @ addTrafficLog in TrafficAnalysisXMLDB");
			}
		}
	}
	
	private synchronized int findIndex(Element r, long t) {
		List<Element> ll = r.getChildren();
		for (Element e : ll) {
			if(t >= Long.valueOf(e.getAttributeValue("time"))) {
				return r.indexOf(e);
			}
		}
		return ll.size();
	}
	
	public synchronized void xmlParsePrint() {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				System.out.println(e.getAttributeValue("time")+" "+e.getChildText("traffic")+" "+e.getChildText("mark")+" "+e.getChildText("isRead")+""+e.getChildText("redundancy"));
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ xmlParsePrint in TrafficAnalysisXMLDB");
		} catch (IOException e2) {
			System.out.println("IOException @ xmlParsePrint in TrafficAnalysisXMLDB");
		}
	}
	
	public synchronized LinkedList<TrafficLog> getRecentTrafficLog(int bulk) {
		LinkedList<TrafficLog> sll = new LinkedList<TrafficLog>();
		while(true) {
			try {
				SAXBuilder builder = new SAXBuilder();
				InputStream file = new FileInputStream(path);
				Document document = builder.build(file);
				Element root = document.getRootElement();
				List<Element> list = root.getChildren();
				int i = 0;
				for (Element e : list) {
					sll.addLast( new TrafficLog(
							e.getAttributeValue("time"),
							e.getChildText("traffic"),
							e.getChildText("mark"),
							e.getChildText("isRead"),
							e.getChildText("fromid"),
							e.getChildText("toid"),
							e.getChildText("redundancy")));
					i += 1;
					if(i >= bulk) {
						break;
					}
				}
				break;
				
			} catch(JDOMException e1) {
				continue;
			} catch (IOException e2) {
				System.out.println("IOException @ getRecentTrafficLog in TrafficAnalysisXMLDB");
				continue;
			}
		}
		return sll;
	}
	
	public synchronized LinkedList<TrafficLog> getTrafficLog(long time) {
		LinkedList<TrafficLog> sll = new LinkedList<TrafficLog>();
	
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				long t = Long.valueOf(e.getAttributeValue("time"));
				if(t > time) {
					sll.addLast( new TrafficLog(
							e.getAttributeValue("time"),
							e.getChildText("traffic"),
							e.getChildText("mark"),
							e.getChildText("isRead"),
							e.getChildText("fromid"),
							e.getChildText("toid"),
							e.getChildText("redundancy")));
				} else {
					break;
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ getRecentStatus in TrafficAnalysisXMLDB");
		} catch (IOException e2) {
			System.out.println("IOException @ getRecentStatus in TrafficAnalysisXMLDB");
		}
		return sll;
	}
	
	public static void printTrafficLogs(LinkedList<TrafficLog> tl) {
		Iterator<TrafficLog> iterator = tl.iterator();
		TrafficLog ts;
		while(iterator.hasNext()) {
			ts = iterator.next();
			System.out.println(ts.getTime()+" "+ts.getTraffic()+" "+ts.getMark()+" "+ts.getInOut());
		}		
	}
	
	public static void main(String ars[]) throws JDOMException, IOException {
		String dir = "xml/" + "hello";
		
		TrafficAnalysisXMLDB xmlDB = new TrafficAnalysisXMLDB(dir, dir+"/tl.xml", "testTl");
		xmlDB.addTrafficLog(new TrafficLog(123, 345, "p", "n", "1", "2", "n"));
		xmlDB.addTrafficLog(new TrafficLog(234, 345, "p", "n", "1", "2", "r"));
		xmlDB.addTrafficLog(new TrafficLog(345, 345, "p", "n", "1", "2", "n"));
		xmlDB.addTrafficLog(new TrafficLog(124, 345, "p", "n", "1", "2", "r"));
		xmlDB.addTrafficLog(new TrafficLog(257, 345, "p", "n", "1", "2", "r"));
		
		xmlDB.xmlParsePrint();
		//LinkedList<TrafficLog> ll = xmlDB.getRecentTrafficLog(10);
		//xmlDB.printTrafficLogs(ll);
	}
}
