package guinterface;

import guinterface.TrafficAnalysisFrame.DemoPanel;

import java.awt.BorderLayout;
import java.awt.Color;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.ClusteredXYBarRenderer;
import org.jfree.data.time.Millisecond;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.time.Year;
import org.jfree.ui.RectangleInsets;

import toolbox.GossipLog;
import toolbox.TrafficLog;

import database.GossipAnalysisXMLDB;
import database.TrafficAnalysisXMLDB;

public class GossipAnalysisFrame extends JFrame {
	private static final long serialVersionUID = 2010892306L;
	
	private static final int TRACE_NUM = 10;
	private static final int INITBULK = 20;
	private static final int INTERVAL = 3000; //millisecond
	
	private JTabbedPane jtpPanel = new JTabbedPane();
	private GossipPanel gossipPanel = new GossipPanel();
	private RedundancyPanel redundancyPanel = new RedundancyPanel();
	
	private ImageIcon icon = new ImageIcon("image/cuckoo.jpg");
	
	private TrafficAnalysisXMLDB trafficAnalysisXMLDB;
	private GossipAnalysisXMLDB  gossipAnalysisXMLDB;
	
	private long initTime = 0;
	private GossipLog lastLog;
	private long lastShowTime = 0;
	
	public GossipAnalysisFrame(GossipAnalysisXMLDB gaxdb, TrafficAnalysisXMLDB taxdb) {
		super("Gossip Analysis");
		gossipAnalysisXMLDB  = gaxdb;
		trafficAnalysisXMLDB = taxdb; 
        
        jtpPanel.add(gossipPanel, "Gossip Messages");
        jtpPanel.add(redundancyPanel, "Redundancy Ratio");
        setContentPane(jtpPanel);
        
        initTime = new Date().getTime();
        //**lastLog = gossipAnalysisXMLDB.getLastGossipLog();
        
        setVisible(true);
        pack();
        setIconImage(icon.getImage());
        setLocationRelativeTo(null); // Center the frame
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        //initFigure();
        //monitor();
    }
	
	public void initFigure() {
		LinkedList<TrafficLog> ll = trafficAnalysisXMLDB.getRecentTrafficLog(INITBULK);
		if(ll != null && ll.size() != 0) {
			lastShowTime = ll.get(0).getTime();
			//System.out.println("last time: " + lastShowTime);
			
			long currentTime = initTime - INTERVAL;
			
			Iterator<TrafficLog> iterator = ll.iterator();
			long reTraffic = 0, ovTraffic = 0;
			while(iterator.hasNext()) {
				TrafficLog tlog = iterator.next();
				//System.out.println(tlog.getTime());
				if(tlog.getTime() >= currentTime) {
					if(tlog.getRedundancy().compareTo("r") == 0) {
						reTraffic += tlog.getTraffic();
					} else {
						ovTraffic += tlog.getTraffic();
					}
				} else if(tlog.getTime() < currentTime) {
					//System.out.println(sTraffic);
					//System.out.println(pTraffic);
					//System.out.println("-------");
					redundancyPanel.addPoint(1, currentTime+INTERVAL, ovTraffic);
					redundancyPanel.addPoint(2, currentTime+INTERVAL, reTraffic);
					currentTime -= INTERVAL;
				}
			}
		}
	}
	
	//using thread otherwise cannot repaint
	public void monitor() {
		new Thread() {
            
            public void run() {
            	xmonitor();
            }
            
		}.start();
	}
	
	//Periodically check the XML-based DB and see whether there are some current gossip logs
	private void xmonitor() {
		while(true) {
			try {
				Thread.sleep(INTERVAL);
			} catch (InterruptedException e) {
				System.out.println("Interrupted Exception in monitor traffic");
			}
			//////////////the gossip dissemination panel staff///////////////////
			//See whether there is a new log (new gossip)
			//Update the existing logs (only trace the latest 10 (TRACE_NUM) logs)
			LinkedList<GossipLog> nll = gossipAnalysisXMLDB.getGossipLogs(initTime, TRACE_NUM);
			if(nll != null && nll.size() != 0) {
				Iterator<GossipLog> iterator = nll.iterator();
				int i = 0;
				while(iterator.hasNext()) {
					GossipLog gl = iterator.next();
					if(lastLog == null || gl.getTime() > lastLog.getTime()) { //this is a new log, add!
						gossipPanel.addPoint(gl);
					} else { //it is an old log, update!
						gossipPanel.updatePoint(gl, i++);
					}
				}
				lastLog = nll.get(0);
			}
			//////////////the redundancy panel staff///////////////////
			LinkedList<TrafficLog> ll = trafficAnalysisXMLDB.getTrafficLog(lastShowTime);
			long overallTraffic=0, redundantTraffic=0;
			
			if(ll != null && ll.size() != 0) {
				lastShowTime = ll.get(0).getTime();
				Iterator<TrafficLog> iterator = ll.iterator();
			
				while(iterator.hasNext()) {
					TrafficLog tl = iterator.next();
					if(tl.getRedundancy().compareTo("r") == 0) {
						redundantTraffic += tl.getTraffic();
					} else {
						overallTraffic += tl.getTraffic();
					}
				}
			}
			long time = new Date().getTime();
			redundancyPanel.addPoint(1, time, overallTraffic);
			redundancyPanel.addPoint(2, time, redundantTraffic);
		}
	}
	
	//For testing only
	public void randomTest() {
		//gossipPanel.randomTest();
		redundancyPanel.randomTest();
	}
	
	static class GossipPanel extends JPanel {
		private static final long serialVersionUID = 2010892318L;
		
		// The time series data (two time series).
        private TimeSeries sentPeer;
        private TimeSeries recvPeer;
        private TimeSeries livePeer;
        private TimeSeries dummy1;
        private TimeSeries dummy2;
        
        private double lastValue = 20.0;
        
        private int i = 1;
        
        /**
         * Creates a new instance.
         */
        public GossipPanel() {
        	super(new BorderLayout());
            this.sentPeer = new TimeSeries("sent", 	Year.class);
            this.recvPeer = new TimeSeries("recv",  Year.class);
            this.livePeer = new TimeSeries("alive", Year.class);
            this.dummy1   = new TimeSeries("",      Year.class);
            this.dummy2   = new TimeSeries("",      Year.class);
            
            TimeSeriesCollection dataset = new TimeSeriesCollection();
            dataset.addSeries(sentPeer);
            dataset.addSeries(recvPeer);
            dataset.addSeries(livePeer);
            dataset.addSeries(dummy1);
            dataset.addSeries(dummy2);
            
            //JFreeChart chart = ChartFactory.createTimeSeriesChart("Traffic Analysis", "Time", "Value", dataset1, true, true, false);
            JFreeChart chart = ChartFactory.createXYBarChart("Gossip Analysis", "Index", true, "Value", dataset, PlotOrientation.VERTICAL, true, false, false);
            XYPlot plot = (XYPlot) chart.getPlot();
            ClusteredXYBarRenderer r = new ClusteredXYBarRenderer();
            plot.setRenderer(r);
            
            ValueAxis yaxis = plot.getRangeAxis();
            yaxis.setRange(0.0, 30.0);
            
            ChartPanel chartPanel = new ChartPanel(chart);
            add(chartPanel);
            
            chartPanel.setPreferredSize(new java.awt.Dimension(500, 270));
        }
        
        public void addPoint(GossipLog gl) {
        	sentPeer.add(new Year(i), gl.getSent());
        	recvPeer.add(new Year(i), gl.getRecv());
        	livePeer.add(new Year(i), gl.getAlive());
        	dummy1.add(new Year(i), 0.0);
            dummy2.add(new Year(i++), 0.0);         
        }
        
        //d is the distance between the tail of the timeSeries to the current update time point 
        public void updatePoint(GossipLog gl, int d) {
        	int index = sentPeer.getItemCount() - 1 - d;
        	//Only need to modify the recv part
        	sentPeer.update(index, gl.getSent());
        	recvPeer.update(index, gl.getRecv());
        }
        
        //For testing only
        public void randomTest() {
        	while(true) {
        		double factor1 = 0.90 + 0.2 * Math.random();
                int x1 = (int) (lastValue * factor1);
                //sentPeer.add(new Year(i), this.lastValue);
                
                double factor2 = 0.90 + 0.2 * Math.random();
                int x2 = (int) (x1 * factor2);
                //recvPeer.add(new Year(i), this.lastValue);
                
                double factor3 = 0.90 + 0.2 * Math.random();
                int x3 = (int) (x2 * factor3);
                //livePeer.add(new Year(i), this.lastValue);
                
                //dummy1.add(new Year(i), 0.0);
                //dummy2.add(new Year(i++), 0.0);
                addPoint(new GossipLog(i++, "", x1, x2, x3));
                
                try {
                	Thread.sleep(500);
                } catch(Exception e) {}
        	}
        }
	}
	
	static class RedundancyPanel extends JPanel {
		private static final long serialVersionUID = 201008162216L;
		
		// The time series data (two time series).
        private TimeSeries overallSeries;
        private TimeSeries redundancySeries;
        
        // The most recent value added. Two are the same
        private double lastValue = 100.0;
        
        /**
         * Creates a new instance.
         */
        public RedundancyPanel() {
            super(new BorderLayout());
            this.overallSeries    = new TimeSeries("Overall traffic", Millisecond.class);
            this.redundancySeries = new TimeSeries("Redundant traffic",  Millisecond.class);
            
            //TimeSeriesCollection dataset1 = new TimeSeriesCollection(this.overallSeries);
            //TimeSeriesCollection dataset2 = new TimeSeriesCollection(this.redundancySeries);
            
            TimeSeriesCollection dataset = new TimeSeriesCollection();
            dataset.addSeries(this.overallSeries);
            dataset.addSeries(this.redundancySeries);
            
            JFreeChart chart = ChartFactory.createXYAreaChart("Redundant Ratio", "Time", "Traffic", dataset, PlotOrientation.VERTICAL, true, true, false);
            XYPlot plot = (XYPlot) chart.getPlot();
            plot.setBackgroundPaint(Color.lightGray);
            plot.setDomainGridlinePaint(Color.white);
            plot.setRangeGridlinePaint(Color.white);
            plot.setAxisOffset(new RectangleInsets(4, 4, 4, 4));
            ValueAxis axis = plot.getDomainAxis();
            axis.setAutoRange(true);

            /*plot.setDataset(1, dataset2);
            NumberAxis rangeAxis2 = new NumberAxis("");
            rangeAxis2.setAutoRangeIncludesZero(false);
            plot.setRenderer(1, new DefaultXYItemRenderer());
            //plot.setRangeAxis(1, rangeAxis2);
            plot.mapDatasetToRangeAxis(1, 0);*/
            
            ChartPanel chartPanel = new ChartPanel(chart);
            add(chartPanel);
            
            chartPanel.setPreferredSize(new java.awt.Dimension(500, 270));
        }
        
        public void addPoint(TrafficLog tl) {
        	if(tl.getRedundancy().compareTo("r") == 0) {
        		this.redundancySeries.add(new Millisecond(new Date(tl.getTime())), tl.getTraffic());
        	} else {
        		this.overallSeries.add(new Millisecond(new Date(tl.getTime())), tl.getTraffic());
        	}
        }
        
        //mark == 1: overall traffic
        //mark == 2: redundant traffic
        public void addPoint(int mark, long time, long traffic) {
        	if(mark == 1) {
        		this.overallSeries.add(new Millisecond(new Date(time)), traffic);
        	} else if(mark == 2) {
        		this.redundancySeries.add(new Millisecond(new Date(time)), traffic);
        	}
        }
        
        //For testing only
        public void randomTest() {
        	
        	while(true) {
        		double factor1 = 0.90 + 0.2 * Math.random();
                this.lastValue = this.lastValue * factor1;
                //Millisecond now1 = new Millisecond();
                //System.out.println("Now = " + now1.toString());
                this.redundancySeries.add(new Millisecond(), this.lastValue);
                
                double factor2 = 0.90 + 0.2 * Math.random();
                this.lastValue = this.lastValue * factor2;
                //Millisecond now2 = new Millisecond();
                //System.out.println("Now = " + now2.toString());
                this.overallSeries.add(new Millisecond(), this.lastValue);
                
                try {
                	Thread.sleep(500);
                } catch(Exception e) {
                	
                }
        	}
        }
	}
    
	public static JPanel createDemoPanel() {
        return new DemoPanel();
    }
	
	public static void main(String[] args) {

    	//TrafficAnalysisFrame demo = new TrafficAnalysisFrame(null);
        //demo.pack();
        //RefineryUtilities.centerFrameOnScreen(demo);
        //demo.setVisible(true);
        //demo.randomTest();
    	//Date date = new Date(12345);
    	//System.out.println(new Millisecond(date));
    	/*String dir = "xml";
    	GossipAnalysisXMLDB gossipDB = new GossipAnalysisXMLDB(dir, dir+"/gossipanalysis.xml", "gossipanalysis");
    	GossipAnalysisFrame gaFrame = new GossipAnalysisFrame(gossipDB);
    	gaFrame.xmonitor();
    	*/
    	/*
    	long lastValue = 100;
    	String mark;
    	for(int i=0; i<100; i++) {
    		double factor = 0.90 + 0.2 * Math.random();
    		lastValue = (long) (lastValue * factor);
    		double markFactor = Math.random();
    		mark = markFactor<0.5 ? "p" : "s";
    		trafficDB.addTrafficLog(new TrafficLog(new Date().getTime(), lastValue, mark, "n"));
    		try {
            	Thread.sleep(500);
            } catch(Exception e) {}
    	}*/
    	
    	GossipAnalysisFrame gaFrame = new GossipAnalysisFrame(null, null);
    	gaFrame.randomTest();
    }
}
