package guinterface;

import java.awt.AWTException;
import java.awt.Component;
import java.awt.Container;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.SystemTray;
import java.awt.TrayIcon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import org.jfree.ui.RefineryUtilities;

import application.Cuckoo;

import rice.p2p.commonapi.Node;
import rice.pastry.PastryNodeFactory;
import winterwell.jtwitter.OAuthSignpostClient;
import winterwell.jtwitter.Twitter;

public class TimelineFrame extends JFrame{
	private static final long serialVersionUID = 10250320101629L;
	
	private TimelinePanel followeeTimelinePanel;
	private TimelinePanel myTimelinePanel;
	private UserPanel followeePanel;
	private UserPanel followerPanel;
	private SearchPanel profilePanel;
	private JTabbedPane jtpPanel = new JTabbedPane();
	
	private JMenu fileMenu = new JMenu("File");
	private JMenu settingsMenu = new JMenu("Settings");
	private JMenu measurementMenu = new JMenu("Measurement");
	private JMenu debugMenu = new JMenu("Debug");
	private JMenu helpMenu = new JMenu("Help");
	
	private JMenu uploadMenu = new JMenu("Upload option");
	private JMenu gossipMenu = new JMenu("Dissemination option"); 
	
	private JMenuItem showMyProfileMenuItem  = new JMenuItem("Show my profile");
	private JMenuItem ModifyProfileMenuItem  = new JMenuItem("Modify my profile");
	private JMenuItem ClearFolloweeStatusMenuItem =new JMenuItem("Clear followee statuses");
	private JMenuItem ClearFollowerStatusMenuItem =new JMenuItem("Clear follower statuses");
	private JMenuItem ReloadFolloweeMenuItem = new JMenuItem("Reload followees");
	private JMenuItem ReloadFollowerMenuItem = new JMenuItem("Reload followers");
	
	private JMenuItem genPartnerMenuItem = new JMenuItem("Generate Partners");
	private JMenuItem followeeHealthItem = new JMenuItem("Print alive followees' health"); 
	private JMenuItem followerHealthItem = new JMenuItem("Print alive followers' health");
	private JMenuItem partnerMapItem     = new JMenuItem("Print partner map");
	
	private JMenuItem trafficMenuItem = new JMenuItem("Traffic analysis");
	private JMenuItem gossipMenuItem  = new JMenuItem("Gossip analysis");
	
	private JMenuItem whoWeAreMenuItem = new JMenuItem("Who we are");
	
	private JRadioButtonMenuItem noButtonMenuItem;
	private JRadioButtonMenuItem twButtonMenuItem;
	private JRadioButtonMenuItem directButtonMenuItem;
	private JRadioButtonMenuItem gossipButtonMenuItem;
	
	private SystemTray systemTray;
	private TrayIcon trayIcon;
	
	private TweetPanel tweetPanel;
	private FunctionPanel functionPanel;
	private ImageIcon icon = new ImageIcon("image/cuckoo.jpg");
	private ImageIcon sIcon = new ImageIcon("image/cuckoo_small.jpg");
	private Cuckoo cuckoo;
	
	private Twitter twitter;
	
	public TimelineFrame(PastryNodeFactory factory, Node node, String uid, String uname) {
		// First, OAuth to login: Make an oauth client
        OAuthSignpostClient oauthClient = new OAuthSignpostClient(OAuthSignpostClient.JTWITTER_OAUTH_KEY, OAuthSignpostClient.JTWITTER_OAUTH_SECRET, "oob");
        // open the authorisation page in the user's browser
        oauthClient.authorizeDesktop(); // Note: this only works on desktop PCs
        // or direct the user to the webpage given jby oauthClient.authorizeUrl() 
        // get the pin from the user since we're using "oob" instead of a callback servlet
        String v = OAuthSignpostClient.askUser("Please enter the verification PIN from Twitter");
        oauthClient.setAuthorizationCode(v);

        // Make a Twitter object
        twitter = new Twitter(uname, oauthClient); 
		
		cuckoo = new Cuckoo(factory, node, uid, this, twitter);
		
		followeeTimelinePanel = new TimelinePanel(cuckoo);
		myTimelinePanel = new TimelinePanel(cuckoo);
		followeePanel = new UserPanel(cuckoo);
		followerPanel = new UserPanel(cuckoo);
		profilePanel  = new SearchPanel(twitter);
		
		tweetPanel    = new TweetPanel(cuckoo);
		functionPanel = new FunctionPanel(cuckoo, tweetPanel, followeeTimelinePanel, profilePanel);
		
		if(cuckoo.getGossipOrNot() == true) {
			directButtonMenuItem = new JRadioButtonMenuItem("direct");
			gossipButtonMenuItem = new JRadioButtonMenuItem("gossip", true);
		} else {
			directButtonMenuItem = new JRadioButtonMenuItem("direct", true);
			gossipButtonMenuItem = new JRadioButtonMenuItem("gossip");
		}
		
		if(cuckoo.getUploadOrNot() == true) {
			noButtonMenuItem = new JRadioButtonMenuItem("None");
			twButtonMenuItem = new JRadioButtonMenuItem("Twitter", true);
		} else {
			noButtonMenuItem = new JRadioButtonMenuItem("None", true);
			twButtonMenuItem = new JRadioButtonMenuItem("Twitter");
		}
		
		systemTray = SystemTray.getSystemTray();
		try {
            trayIcon = new TrayIcon(sIcon.getImage(),"Cuckoo");
            systemTray.add(trayIcon);
        }
        catch (AWTException e2) {e2.printStackTrace();}
        trayIcon.addMouseListener(
        		new MouseAdapter() {
        			public void mouseClicked(MouseEvent e) {
        				if(e.getClickCount() == 2)//double click: show the frame again
                            setExtendedState(Frame.NORMAL);//״̬
                            setVisible(true);
                    }
                }
        );
		
		JMenuBar jmb = new JMenuBar();
		setJMenuBar(jmb);
		jmb.add(fileMenu);
		jmb.add(settingsMenu);
		jmb.add(measurementMenu);
		jmb.add(debugMenu);
		jmb.add(helpMenu);
		
		measurementMenu.add(trafficMenuItem);
		measurementMenu.add(gossipMenuItem);
		
		settingsMenu.add(uploadMenu);
		settingsMenu.add(gossipMenu);
	
		uploadMenu.add(noButtonMenuItem);
		uploadMenu.add(twButtonMenuItem);
		
		gossipMenu.add(directButtonMenuItem);
		gossipMenu.add(gossipButtonMenuItem);
		
		ButtonGroup btg1 = new ButtonGroup();
		btg1.add(noButtonMenuItem);
		btg1.add(twButtonMenuItem);
		
		ButtonGroup btg2 = new ButtonGroup();
		btg2.add(directButtonMenuItem);
		btg2.add(gossipButtonMenuItem);
		
		fileMenu.add(showMyProfileMenuItem);
		fileMenu.add(ModifyProfileMenuItem);
		fileMenu.add(ClearFolloweeStatusMenuItem);
		fileMenu.add(ClearFollowerStatusMenuItem);
		fileMenu.add(ReloadFolloweeMenuItem);
		fileMenu.add(ReloadFollowerMenuItem);
		
		debugMenu.add(genPartnerMenuItem);
		debugMenu.add(followeeHealthItem);
		debugMenu.add(followerHealthItem);
		debugMenu.add(partnerMapItem);
		
		helpMenu.add(whoWeAreMenuItem);
		
		noButtonMenuItem.addActionListener(new NoneUploadActionListener());
		twButtonMenuItem.addActionListener(new Upload2TwitterActionListener());
		
		directButtonMenuItem.addActionListener(new DirectDisseminateActionListener());
		gossipButtonMenuItem.addActionListener(new GossipDisseminateActionListener());
		
		trafficMenuItem.addActionListener(new TrafficAnalysisActionListener());
		gossipMenuItem.addActionListener(new GossipAnalysisActionListener());
		
		showMyProfileMenuItem.addActionListener(new ShowMyProfileActionListener());
		ModifyProfileMenuItem.addActionListener(new ModifyProfileActionListener());
		ClearFolloweeStatusMenuItem.addActionListener(new ClearFolloweeStatusActionListener());
		ClearFollowerStatusMenuItem.addActionListener(new ClearFollowerStatusActionListener());
		ReloadFolloweeMenuItem.addActionListener(new ReloadFolloweeActionListener());
		ReloadFollowerMenuItem.addActionListener(new ReloadFollowerActionListener());
		
		genPartnerMenuItem.addActionListener(new GenPartnersActionListener());
		followeeHealthItem.addActionListener(new FolloweeHealthActionListener());
		followerHealthItem.addActionListener(new FollowerHealthActionListener());
		partnerMapItem.addActionListener(new PartnerMapActionListener());
		
		whoWeAreMenuItem.addActionListener(new HelpActionListener());
		
	    setSize(390, 550);
	    setIconImage(icon.getImage());
	    setLayout(new GridBagLayout());
	    setTitle("Cuckoo");
		setLocationRelativeTo(null); // Center the frame
	    
		JScrollPane scrollFolloweeTimelinePane = new JScrollPane(followeeTimelinePanel);
		scrollFolloweeTimelinePane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scrollFolloweeTimelinePane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		
		JScrollPane scrollMyTimelinePane = new JScrollPane(myTimelinePanel);
		scrollMyTimelinePane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scrollMyTimelinePane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);

		JScrollPane scrollProfilePane = new JScrollPane(profilePanel);
		scrollProfilePane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scrollProfilePane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		
		JScrollPane scrollFloweePane = new JScrollPane(followeePanel);
		scrollFloweePane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scrollFloweePane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		
		JScrollPane scrollFollowerPane = new JScrollPane(followerPanel);
		scrollFollowerPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scrollFollowerPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		
	    jtpPanel.add(scrollFolloweeTimelinePane, "Home");
	    jtpPanel.add(scrollMyTimelinePane, "Profile");
	    jtpPanel.add(scrollFloweePane, "Followee");
	    jtpPanel.add(scrollFollowerPane, "Follower");
	    jtpPanel.add(scrollProfilePane, "Search");
	    
	    followeeTimelinePanel.showFolloweeRecentStatus();
	    myTimelinePanel.showMyRecentStatus();
	    followeePanel.showFolloweeProfile();
	    followerPanel.showFollowerProfile();
	    
	    GridBagConstraints gbConstraints = new GridBagConstraints();
	    gbConstraints.fill = GridBagConstraints.BOTH;
	    gbConstraints.anchor = GridBagConstraints.CENTER;
	    Container container = getContentPane();
	    addComp(jtpPanel,      container, gbConstraints, 0,  0, 6, 8, 10, 10);
	    addComp(tweetPanel,    container, gbConstraints, 8,  0, 1, 6, 1,  1 );
	    addComp(functionPanel, container, gbConstraints, 10, 0, 2, 6, 0,  0 );
	    
	    //update offline status + follower msg
	    cuckoo.fetchTweetFromServer();
	    cuckoo.updateFollowerProfile();
	    cuckoo.updateFolloweeProfile();
	    
	    addWindowListener(new TimeLineFrameWindowListener());
	    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	}
	
	public void appendFolloweeStatus(String text) {
		followeeTimelinePanel.add2textarea(text);
	}
	
	public void appendMyStatus(String text) {
		myTimelinePanel.add2textarea(text);
	}
	
	public void append2FolloweePanel(String text) {
		followeePanel.add2textarea(text);
	}
	
	public void append2FollowerPanel(String text) {
		if(followerPanel == null) {
			System.out.println("???");
		}
		followerPanel.add2textarea(text);
	}
	
	private void addComp(Component c, Container container,
			            GridBagConstraints gbConstraints, 
			            int row, int column,
			            int numberOfRows, int numberOfColumns,
			            double weightx, double weighty) {

		gbConstraints.gridx = column;
		gbConstraints.gridy = row;
		gbConstraints.gridwidth = numberOfColumns;
		gbConstraints.gridheight = numberOfRows;
		gbConstraints.weightx = weightx;
		gbConstraints.weighty = weighty;

		container.add(c, gbConstraints); 
	}
	
	public void showWarningDialog(String text) {
		javax.swing.JOptionPane.showMessageDialog(null, text); 
	}
	
	private class TimeLineFrameWindowListener implements WindowListener {
		
		public void windowActivated(WindowEvent arg0) {}

		public void windowClosed(WindowEvent arg0) {}

		public void windowClosing(WindowEvent arg0) {
			cuckoo.quit();
		}

		public void windowDeactivated(WindowEvent arg0) {}

		public void windowDeiconified(WindowEvent arg0) {}

		public void windowIconified(WindowEvent arg0) {
			dispose();
		}

		public void windowOpened(WindowEvent arg0) {}
		
	}
	
	private class NoneUploadActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			cuckoo.setUploadOrNot(false);
			System.out.println("cuckoo.setUploadOrNot(false);");
		}
	}
	
	private class Upload2TwitterActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			cuckoo.setUploadOrNot(true);
			System.out.println("cuckoo.setUploadOrNot(true);");
		}
	}
	
	private class GossipDisseminateActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			cuckoo.setGossipOrNot(true);
			System.out.print("cuckoo.GossipOrNot --> ");
			cuckoo.printGossipOrNot();
		}
	}
	
	private class DirectDisseminateActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			cuckoo.setGossipOrNot(false);
			System.out.print("cuckoo.GossipOrNot --> ");
			cuckoo.printGossipOrNot();
		}
	}
	
	private class TrafficAnalysisActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			new Thread() {
	            public void run() {
	            	//trigger the traffic analysis frame
	            	TrafficAnalysisFrame trafficFrame = new TrafficAnalysisFrame(cuckoo.getTrafficAnalysisXMLDB());
	            	trafficFrame.pack();
	            	RefineryUtilities.centerFrameOnScreen(trafficFrame);
	            	trafficFrame.setVisible(true);
	            	//trafficFrame.setAlwaysOnTop(true);
	            	trafficFrame.initFigure();
	            	trafficFrame.monitor();
	            	//trafficFrame.randomTest();
	            }
			}.start();
		}
	}
	
	private class GossipAnalysisActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			new Thread() {
	            public void run() {
	            	//trigger the gossip analysis frame
	            	GossipAnalysisFrame gossipFrame = new GossipAnalysisFrame(cuckoo.getGossipAnalysisXMLDB(), cuckoo.getTrafficAnalysisXMLDB());
	            	gossipFrame.pack();
	            	RefineryUtilities.centerFrameOnScreen(gossipFrame);
	            	gossipFrame.setVisible(true);
	            	//gossipFrame.setAlwaysOnTop(true);
	            	gossipFrame.initFigure();
	            	gossipFrame.monitor();
	            	//trafficFrame.randomTest();
	            }
			}.start();
		}
	}
	
	private class GenPartnersActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			cuckoo.generatePartners();
		}
	}
	
	private class FolloweeHealthActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			cuckoo.printFolloweeHealthIndex();
		}
	}
	
	private class FollowerHealthActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			cuckoo.printFollowerHealthIndex();
		}
	}
	
	private class PartnerMapActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			cuckoo.printPartnerMap();
		}
	}
	
	private class ShowMyProfileActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			HeadFrame headFrame = new HeadFrame(cuckoo);
			headFrame.setVisible(true);
		}
	}
	
	private class ModifyProfileActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			//TODO
		}
	}
	
	private class ClearFolloweeStatusActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			followeeTimelinePanel.clear();
		}
	}
	
	private class ClearFollowerStatusActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			myTimelinePanel.clear();
		}
	}
	
	private class ReloadFolloweeActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			followeePanel.clear();
			followeePanel.showFolloweeProfile();
		}
	}
	
	private class ReloadFollowerActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			followerPanel.clear();
			followerPanel.showFollowerProfile();
		}
	}
	
	private class HelpActionListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			HelpFrame helpFrame = new HelpFrame();
			helpFrame.setVisible(true);
		}
	}
} 
