package guinterface;

import java.awt.BorderLayout;
import java.awt.Color;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.DefaultXYItemRenderer;
import org.jfree.data.time.Millisecond;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.ui.RectangleInsets;

import database.TrafficAnalysisXMLDB;

import toolbox.TrafficLog;


public class TrafficAnalysisFrame extends JFrame {
	private static final long serialVersionUID = 2010762152L;
	
	private static final int INITBULK = 20;
	private static final int INTERVAL = 3000; //millisecond
	
	private JTabbedPane jtpPanel = new JTabbedPane();
	
	private DemoPanel incomingDP = new DemoPanel();
	private DemoPanel outgoingDP = new DemoPanel();
	
	private JTextArea incomingTrafficTrace = new JTextArea();
	private JTextArea outgoingTrafficTrace = new JTextArea();
	
	private ImageIcon icon = new ImageIcon("image/cuckoo.jpg");
	
	private TrafficAnalysisXMLDB trafficAnalysisXMLDB;
	private long initTime = 0;
	private long lastShowTime = 0;
	
	public TrafficAnalysisFrame(TrafficAnalysisXMLDB taxdb) {
        super("Traffic Analysis");
        trafficAnalysisXMLDB = taxdb;
        
        jtpPanel.add(incomingDP, "Incoming Traffic");
        jtpPanel.add(incomingTrafficTrace, "Incoming Trace");
        jtpPanel.add(outgoingDP, "Outcoming Traffic");
        jtpPanel.add(outgoingTrafficTrace, "Outgoing Trace");
        setContentPane(jtpPanel);
        
        initTime = new Date().getTime();
        setVisible(true);
        pack();
        setIconImage(icon.getImage());
        setLocationRelativeTo(null); // Center the frame
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        //initFigure();
        //monitor();
    }
	
	//1. fetch INITBULK number of traffic logs from the XML-based DB and show on the figure
	//2. record the latest time;
	/*
	public void initFigure() {
		LinkedList<TrafficLog> ll = trafficAnalysisXMLDB.getRecentTrafficLog(INITBULK);
		lastShowTime = ll.get(0).getTime();
		System.out.println("last time: " + lastShowTime);
		if(ll != null) {
			Iterator<TrafficLog> iterator = ll.iterator();
			while(iterator.hasNext()) {
				dp.addPoint(iterator.next());
			}
		}
	}*/
	
	//1. fetch INITBULK number of traffic logs from the XML-based DB
	//2. get the periodical view 
	public void initFigure() {
		LinkedList<TrafficLog> ll = trafficAnalysisXMLDB.getRecentTrafficLog(INITBULK);
		if(ll != null && ll.size() != 0) {
			lastShowTime = ll.get(0).getTime();
			//System.out.println("last time: " + lastShowTime);
			
			long currentTime = initTime - INTERVAL;
			
			Iterator<TrafficLog> iterator = ll.iterator();
			long isTraffic = 0, ipTraffic = 0;
			long osTraffic = 0, opTraffic = 0;
			long fwdTraffic = 0;
			while(iterator.hasNext()) {
				TrafficLog tlog = iterator.next();
				//System.out.println(tlog.getTime());
				
				if(tlog.getMark().compareTo("p") == 0 && tlog.getInOut().compareTo("i") == 0) {
					incomingTrafficTrace.insert("peer "+tlog.getTraffic()+"B, from node ["+tlog.getFromId()+"] to node ["+tlog.getToId()+"]\n", 0);
				} else if(tlog.getMark().compareTo("s") == 0 && tlog.getInOut().compareTo("i") == 0) {
					incomingTrafficTrace.insert("serv "+tlog.getTraffic()+"B, from node ["+tlog.getFromId()+"] to node ["+tlog.getToId()+"]\n", 0);
				} else if(tlog.getMark().compareTo("p") == 0 && tlog.getInOut().compareTo("o") == 0) {
					outgoingTrafficTrace.insert("peer "+tlog.getTraffic()+"B, from node ["+tlog.getFromId()+"] to node ["+tlog.getToId()+"]\n", 0);
				} else if(tlog.getMark().compareTo("s") == 0 && tlog.getInOut().compareTo("o") == 0) {
					outgoingTrafficTrace.insert("serv "+tlog.getTraffic()+"B, from node ["+tlog.getFromId()+"] to node ["+tlog.getToId()+"]\n", 0);
				} else if(tlog.getMark().compareTo("f") == 0) {
					incomingTrafficTrace.insert("fwrd "+tlog.getTraffic()+"B, from node ["+tlog.getFromId()+"] to node ["+tlog.getToId()+"]\n", 0);
					outgoingTrafficTrace.insert("fwrd "+tlog.getTraffic()+"B, from node ["+tlog.getFromId()+"] to node ["+tlog.getToId()+"]\n", 0);
				}
				
				if(tlog.getTime() >= currentTime) {
					if(tlog.getMark().compareTo("p") == 0 && tlog.getInOut().compareTo("i") == 0) {
						ipTraffic += tlog.getTraffic();
					} else if(tlog.getMark().compareTo("s") == 0 && tlog.getInOut().compareTo("i") == 0) {
						isTraffic += tlog.getTraffic();
					} else if(tlog.getMark().compareTo("p") == 0 && tlog.getInOut().compareTo("o") == 0) {
						opTraffic += tlog.getTraffic();
					} else if(tlog.getMark().compareTo("s") == 0 && tlog.getInOut().compareTo("o") == 0) {
						osTraffic += tlog.getTraffic();
					} else if(tlog.getMark().compareTo("f") == 0) {
						fwdTraffic += tlog.getTraffic();
					}
				} else if(tlog.getTime() < currentTime) {
					//System.out.println(sTraffic);
					//System.out.println(pTraffic);
					//System.out.println("-------");
					
					incomingDP.addPoint(1, currentTime+INTERVAL, isTraffic);
					incomingDP.addPoint(2, currentTime+INTERVAL, ipTraffic);
					incomingDP.addPoint(3, currentTime+INTERVAL, fwdTraffic);
					
					outgoingDP.addPoint(1, currentTime+INTERVAL, osTraffic);
					outgoingDP.addPoint(2, currentTime+INTERVAL, opTraffic);
					outgoingDP.addPoint(3, currentTime+INTERVAL, fwdTraffic);
					
					isTraffic = 0; 
					ipTraffic = 0;
					osTraffic = 0;
					opTraffic = 0;
					fwdTraffic = 0;
					currentTime -= INTERVAL;
					
					if(tlog.getMark().compareTo("p") == 0 && tlog.getInOut().compareTo("i") == 0) {
						ipTraffic += tlog.getTraffic();
					} else if(tlog.getMark().compareTo("s") == 0 && tlog.getInOut().compareTo("i") == 0) {
						isTraffic += tlog.getTraffic();
					} else if(tlog.getMark().compareTo("p") == 0 && tlog.getInOut().compareTo("o") == 0) {
						opTraffic += tlog.getTraffic();
					} else if(tlog.getMark().compareTo("s") == 0 && tlog.getInOut().compareTo("o") == 0) {
						osTraffic += tlog.getTraffic();
					} else if(tlog.getMark().compareTo("f") == 0) {
						fwdTraffic += tlog.getTraffic();
					}
				}
			}
		}
	}
	
	//using thread otherwise cannot repaint
	public void monitor() {
		new Thread() {
            
            public void run() {
            	xmonitor();
            }
            
		}.start();
	}
	
	//Periodically check the XML-based DB and see whether there are some current traffic logs 
	private void xmonitor() {
		while(true) {
			try {
				Thread.sleep(INTERVAL);
			} catch (InterruptedException e) {
				System.out.println("Interrupted Exception in monitor traffic");
			}
			
			LinkedList<TrafficLog> ll = trafficAnalysisXMLDB.getTrafficLog(lastShowTime);
			long inPeerTraffic=0, inServerTraffic=0, outPeerTraffic = 0, outServerTraffic = 0, fwdTraffic = 0;
			
			if(ll != null && ll.size() != 0) {
				lastShowTime = ll.get(0).getTime();
				Iterator<TrafficLog> iterator = ll.iterator();
			
				while(iterator.hasNext()) {
					TrafficLog tl = iterator.next();
					if(tl.getMark().compareTo("p") == 0 && tl.getInOut().compareTo("i") == 0) {
						inPeerTraffic += tl.getTraffic();
						incomingTrafficTrace.insert("peer "+tl.getTraffic()+"B, from node ["+tl.getFromId()+"] to node ["+tl.getToId()+"]\n", 0);
					} else if(tl.getMark().compareTo("s") == 0 && tl.getInOut().compareTo("i") == 0) {
						inServerTraffic += tl.getTraffic();
						incomingTrafficTrace.insert("serv "+tl.getTraffic()+"B, from node ["+tl.getFromId()+"] to node ["+tl.getToId()+"]\n", 0);
					} else if(tl.getMark().compareTo("p") == 0 && tl.getInOut().compareTo("o") == 0) {
						outPeerTraffic += tl.getTraffic();
						outgoingTrafficTrace.insert("peer "+tl.getTraffic()+"B, from node ["+tl.getFromId()+"] to node ["+tl.getToId()+"]\n", 0);
					} else if(tl.getMark().compareTo("s") == 0 && tl.getInOut().compareTo("o") == 0) {
						outServerTraffic += tl.getTraffic();
						outgoingTrafficTrace.insert("serv "+tl.getTraffic()+"B, from node ["+tl.getFromId()+"] to node ["+tl.getToId()+"]\n", 0);
					} else if(tl.getMark().compareTo("f") == 0) {
						fwdTraffic += tl.getTraffic();
						incomingTrafficTrace.insert("fwrd "+tl.getTraffic()+"B, from node ["+tl.getFromId()+"] to node ["+tl.getToId()+"]\n", 0);
						outgoingTrafficTrace.insert("fwrd "+tl.getTraffic()+"B, from node ["+tl.getFromId()+"] to node ["+tl.getToId()+"]\n", 0);
					}
				}
			}
			long time = new Date().getTime();
			incomingDP.addPoint(1, time, inServerTraffic);
			incomingDP.addPoint(2, time, inPeerTraffic);
			outgoingDP.addPoint(1, time, outServerTraffic);
			outgoingDP.addPoint(2, time, outPeerTraffic);
			incomingDP.addPoint(3, time, fwdTraffic);
			outgoingDP.addPoint(3, time, fwdTraffic);
		}
	}
	
	//For testing only
	public void randomTest() {
		incomingDP.randomTest();
	}
	
	static class DemoPanel extends JPanel {
		private static final long serialVersionUID = 2010762246L;
		
		// The time series data (three time series).
        private TimeSeries serverSeries;
        private TimeSeries forwardSeries;
        private TimeSeries peerSeries;
        
        // The most recent value added. Two are the same
        private double lastValue = 100.0;
        
        /**
         * Creates a new instance.
         */
        public DemoPanel() {
            super(new BorderLayout());
            this.serverSeries  = new TimeSeries("Traffic from Server", Millisecond.class);
            this.peerSeries    = new TimeSeries("Traffic from Peers",  Millisecond.class);
            this.forwardSeries = new TimeSeries("Forwarding Traffic",  Millisecond.class);
            
            TimeSeriesCollection dataset1 = new TimeSeriesCollection(this.serverSeries);
            TimeSeriesCollection dataset2 = new TimeSeriesCollection(this.peerSeries);
            TimeSeriesCollection dataset3 = new TimeSeriesCollection(this.forwardSeries);
            
            JFreeChart chart = ChartFactory.createTimeSeriesChart("Traffic Analysis", "Time", "Value", dataset1, true, true, false);
            XYPlot plot = (XYPlot) chart.getPlot();
            plot.setBackgroundPaint(Color.lightGray);
            plot.setDomainGridlinePaint(Color.white);
            plot.setRangeGridlinePaint(Color.white);
            plot.setAxisOffset(new RectangleInsets(4, 4, 4, 4));
            ValueAxis axis = plot.getDomainAxis();
            axis.setAutoRange(true);

            plot.setDataset(1, dataset2);
            NumberAxis rangeAxis2 = new NumberAxis("");
            rangeAxis2.setAutoRangeIncludesZero(false);
            plot.setRenderer(1, new DefaultXYItemRenderer());
            //plot.setRangeAxis(1, rangeAxis2);
            plot.mapDatasetToRangeAxis(1, 0);
            
            plot.setDataset(2, dataset3);
            NumberAxis rangeAxis3 = new NumberAxis("");
            rangeAxis3.setAutoRangeIncludesZero(false);
            plot.setRenderer(2, new DefaultXYItemRenderer());
            plot.mapDatasetToRangeAxis(2, 0);
            
            ChartPanel chartPanel = new ChartPanel(chart);
            add(chartPanel);
            
            chartPanel.setPreferredSize(new java.awt.Dimension(500, 270));
        }
        
        public void addPoint(TrafficLog tl) {
        	if(tl.getMark().compareTo("s") == 0) {
        		this.serverSeries.add(new Millisecond(new Date(tl.getTime())), tl.getTraffic());
        	} else if(tl.getMark().compareTo("p") == 0) {
        		this.peerSeries.add(new Millisecond(new Date(tl.getTime())), tl.getTraffic());
        	} else if(tl.getMark().compareTo("f") == 0) {
        		this.forwardSeries.add(new Millisecond(new Date(tl.getTime())), tl.getTraffic());
        	}
        }
        
        //mark == 1: server traffic
        //mark == 2: peer traffic
        public void addPoint(int mark, long time, long traffic) {
        	if(mark == 1) {
        		this.serverSeries.add(new Millisecond(new Date(time)), traffic);
        	} else if(mark == 2) {
        		this.peerSeries.add(new Millisecond(new Date(time)), traffic);
        	} else if(mark == 3) {
        		this.forwardSeries.add(new Millisecond(new Date(time)), traffic);
        	}
        }
        
        //For testing only
        public void randomTest() {
        	
        	while(true) {
        		double factor1 = 0.90 + 0.2 * Math.random();
                this.lastValue = this.lastValue * factor1;
                //Millisecond now1 = new Millisecond();
                //System.out.println("Now = " + now1.toString());
                this.serverSeries.add(new Millisecond(), this.lastValue);
                
                double factor2 = 0.90 + 0.2 * Math.random();
                this.lastValue = this.lastValue * factor2;
                //Millisecond now2 = new Millisecond();
                //System.out.println("Now = " + now2.toString());
                this.peerSeries.add(new Millisecond(), this.lastValue);
                
                double factor3 = 0.90 + 0.2 * Math.random();
                this.lastValue = this.lastValue * factor3;
                //Millisecond now2 = new Millisecond();
                //System.out.println("Now = " + now2.toString());
                this.forwardSeries.add(new Millisecond(), this.lastValue);
                
                try {
                	Thread.sleep(500);
                } catch(Exception e) {
                	
                }
        	}
        }
	}
    
	public static JPanel createDemoPanel() {
        return new DemoPanel();
    }
	
    /**
     * Starting point for the demonstration application.
     *
     * @param args  ignored.
     */
    public static void main(String[] args) {

    	//TrafficAnalysisFrame demo = new TrafficAnalysisFrame(null);
        //demo.pack();
        //RefineryUtilities.centerFrameOnScreen(demo);
        //demo.setVisible(true);
        //demo.randomTest();
    	//Date date = new Date(12345);
    	//System.out.println(new Millisecond(date));
    	String dir = "xml/" + "0x113601";
    	TrafficAnalysisXMLDB trafficDB = new TrafficAnalysisXMLDB(dir, dir+"/trafficanalysis.xml", "trafficanalysis");
    	
    	/*
    	long lastValue = 100;
    	String mark;
    	for(int i=0; i<100; i++) {
    		double factor = 0.90 + 0.2 * Math.random();
    		lastValue = (long) (lastValue * factor);
    		double markFactor = Math.random();
    		mark = markFactor<0.5 ? "p" : "s";
    		trafficDB.addTrafficLog(new TrafficLog(new Date().getTime(), lastValue, mark, "n"));
    		try {
            	Thread.sleep(500);
            } catch(Exception e) {}
    	}*/
    	
    	TrafficAnalysisFrame taFrame = new TrafficAnalysisFrame(null);
    	
    	//taFrame.initFigure();
    	//taFrame.monitor();
    	taFrame.randomTest();
    }
}
