package message;
import java.io.IOException;

import application.Status;

import rice.p2p.commonapi.Endpoint;
import rice.p2p.commonapi.Id;
import rice.p2p.commonapi.NodeHandle;
import rice.p2p.commonapi.rawserialization.*;
import rice.p2p.util.BloomFilter;

public class GossipUpdateStatusMsg extends CuckooPiggybackMsg {
	private static final long serialVersionUID = 201008071201L;
	public static final short TYPE = 23;
    
    private String statusId;
    private String authorId;
    private String text;
    private long time;
    
    // Constructors.
    public GossipUpdateStatusMsg(Id from, Id to, String fid, String tid, NodeHandle nh, BloomFilter bf, String stId, String authorId, String text, long time) {
    	super(from, to, fid, tid, nh, bf);
    	
    	this.statusId = stId;
    	this.authorId = authorId;
    	this.text = text;
    	this.time = time;
    }
    
    public GossipUpdateStatusMsg(Id from, Id to, String fid, String tid, NodeHandle nh, BloomFilter bf, Status s) {
    	super(from, to, fid, tid, nh, bf);
    	
    	this.statusId = s.getStatusId();
    	this.authorId = s.getAuthorId();
    	this.text = s.getText();
    	this.time = s.getTime();
    }

	public short getType() {
		return TYPE;
	}

	public void serialize(OutputBuffer buf) throws IOException {
		super.serialize(buf);

	    buf.writeUTF(statusId);
	    buf.writeUTF(authorId);
	    buf.writeUTF(text);
	    buf.writeLong(time);
	}

	public GossipUpdateStatusMsg(InputBuffer buf, Endpoint endpoint) throws IOException {
		super(buf, endpoint);
	    
	    statusId = buf.readUTF();
	    authorId = buf.readUTF();
	    text = buf.readUTF();
	    time = buf.readLong();
	}
	
	public String getAuthorId() {
		return authorId;
	}
	
	public String getText() {
		return text;
	}
	
	public long getTime() {
		return time;
	}
	
	public String getStatusId() {
		return statusId;
	}
	
	public String toString() {
		return "GossipUpdateStatusMsg: from " + fromUserId + ", to " + toUserId + ", ---text: " + text; 
	}
}
