package toolbox;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import database.PartnerXMLDB;

public class PartnerMap {
	public static final int NUM_OF_PARTNERS = 5;
	
	public static final int OLD_MAP = 1;
	public static final int NEW_MAP = 2;
	
	//String: followeeId, PartnerQueue: containing partners
	public HashMap<String, PartnerQueue> oldPartnerMap = new HashMap<String, PartnerQueue>();
	public HashMap<String, PartnerQueue> newPartnerMap = new HashMap<String, PartnerQueue>();
	PartnerXMLDB partnerXMLDB;
	
	public PartnerMap(PartnerXMLDB db) {
		partnerXMLDB = db;
		//initial for the oldPartnerMap
		partnerXMLDB.initPartnerList(this);
	}
	
	//Initially add to the oldPartnerMap (only for oldPartnerMap)
	//Used at the beginning of this session
	public void initAdd(String id, Partner p) {
		PartnerQueue opqueue = oldPartnerMap.get(id);
		if(opqueue == null) {
			opqueue = new PartnerQueue(id, this);
			opqueue.addPartner(p);
			oldPartnerMap.put(id, opqueue);
		} else {
			opqueue.addPartner(p);
		}
	}
	
	//Add at runtime (when a routing msg is listened)
	public void runtimeAdd(String id, Partner p) {
		//System.out.println("******************id: " + id + ", p: " + p.getPartnerId());
		PartnerQueue opqueue = oldPartnerMap.get(id);
		if(opqueue == null) {
			opqueue = new PartnerQueue(id, this);
			opqueue.addPartner(p);
			oldPartnerMap.put(id, opqueue);
		} else if(opqueue.getLength() < NUM_OF_PARTNERS) {
			opqueue.addPartner(p);
		} else if(opqueue.contains(p) == true) {
			opqueue.refresh(p); //automatically increase the health
		} else {
			add2NewMap(id, p);
		}
	}
	
	private void add2NewMap(String id, Partner p) {
		PartnerQueue npqueue = newPartnerMap.get(id);
		if(npqueue == null) {
			npqueue = new PartnerQueue(id, this);
			npqueue.addPartner(p);
			newPartnerMap.put(id, npqueue);
		} else if(npqueue.contains(p) == true) {
			npqueue.refresh(p); //automatically increase the health
		} else {
			npqueue.addPartner(p);
		}
	}
	
	//Compete add to oldPartnerMap
	//Used at the end of this session
	private void competeAdd(String id, Partner p) {
		PartnerQueue opqueue = oldPartnerMap.get(id);
		if(opqueue == null) {
			opqueue = new PartnerQueue(id, this);
			opqueue.addPartner(p);
			oldPartnerMap.put(id, opqueue);
		} else {
			opqueue.addPartner(p);
		}
	}
	
	public List<String> getPartners(String id) {
		/*if(oldPartnerMap == null && newPartnerMap == null) {
			return null;
		} else if(oldPartnerMap == null) {
			PartnerQueue pq = newPartnerMap.get(id);
			if(pq == null) {
				return null;
			} else {
				return pq.getPartnerIds();
			}
		} else if(newPartnerMap == null) {
			PartnerQueue pq = oldPartnerMap.get(id);
			if(pq == null) {
				return null;
			} else {
				return pq.getPartnerIds();
			}
		} else {*/
			PartnerQueue npq = newPartnerMap.get(id);
			PartnerQueue opq = oldPartnerMap.get(id);
			
			if(npq == null && opq == null) {
				return null;
			} else if(npq == null) {
				return opq.getPartnerIds();
			} else if(opq == null) {
				return npq.getPartnerIds();
			} else {
				List<String> re = new ArrayList<String>();
				re.addAll(opq.getPartnerIds());
				re.addAll(npq.getPartnerIds());
				return re;
			}
		/*}*/
	}
	
	public void deletePartner(String cfid, String pid) {
		if(oldPartnerMap != null) {
			PartnerQueue pq = oldPartnerMap.get(cfid);
			if(pq != null) {
				pq.deletePartner(pid);
			}
		}
		
		if(newPartnerMap != null) {
			PartnerQueue pq = newPartnerMap.get(cfid);
			if(pq != null) {
				pq.deletePartner(pid);
			}
		}
	}
	
	public void increaseHealth(String pid, String uid) {
		PartnerQueue npq = newPartnerMap.get(pid);
		PartnerQueue opq = oldPartnerMap.get(pid);
		if(npq == null && opq == null) {
			System.out.println("Error in increaseHealth!!!!!!!!!!!!!!!!!!!!!!!");
		} else if(npq == null) {
			opq.increaseHealth(uid);
		} else if(opq == null) {
			npq.increaseHealth(uid);
		}
	}
	
	public void decreaseHealth(String pid, String uid) {
		PartnerQueue npq = newPartnerMap.get(pid);
		PartnerQueue opq = oldPartnerMap.get(pid);
		if(npq == null && opq == null) {
			System.out.println("Error in decreaseHealth!!!!!!!!!!!!!!!!!!!!!!!");
		} else if(npq == null) {
			opq.decreaseHealth(uid);
		} else if(opq == null) {
			npq.decreaseHealth(uid);
		}
	}
	
	//Compare the new partners with the old guys
	public void writeBack() {
		//HashMap<String, PartnerQueue> rePartnerMap = new HashMap<String, PartnerQueue>();
		
		Set<String> keyset = newPartnerMap.keySet();
		Iterator<String> iterator = keyset.iterator();
		while(iterator.hasNext()) {
			String id = iterator.next();
			List<Partner> pl= newPartnerMap.get(id).getPartnerList();
			
			Iterator<Partner> pIterator = pl.iterator();
			while(pIterator.hasNext()) {
				Partner tmpp = pIterator.next();
				competeAdd(id, tmpp);
			}
		}
		//System.out.println("****************************");
		//printPartnerMap(1);
		partnerXMLDB.writeBack(oldPartnerMap);
	}
	
	//i == 1: print oldPartnerMap
	//i == 2: print newPartnerMap
	public void printPartnerMap(int i) {
		HashMap<String, PartnerQueue> partnerMap;
		if(i == 1) {
			partnerMap = oldPartnerMap;
		} else {
			partnerMap = newPartnerMap;
		}
		
		Set<String> keyset = partnerMap.keySet();
		Iterator<String> iterator = keyset.iterator();
		while(iterator.hasNext()) {
			String id = iterator.next();
			System.out.print("followee (" + id + "): ");
			partnerMap.get(id).printQueue();
		}
	}
	
	public void printPartnerMap() {
		System.out.println("Old Partners' Map: --- ");
		printPartnerMap(1);
		System.out.println("New Partners' Map: --- ");
		printPartnerMap(2);
	}
	
	/////////////////////////////////////////////////////////////////////
	//Queue with fixed length
	public class PartnerQueue {
		private PartnerMap partnerMap;
		private String followeeId;
		private ArrayList<Partner> partnerList = new ArrayList<Partner>();
		private int length;
		
		public PartnerQueue(String fid, PartnerMap pm) {
			followeeId = fid;
			length = 0;
			partnerMap = pm;
		}
		
		public void addPartner(Partner p) {
			int index = indexOf(p);
			if(index != -1) {
				//update the partner
				Partner up = partnerList.get(index);
				up.refresh(p);
			} else {
				if(partnerList.size() < NUM_OF_PARTNERS) {
					partnerList.add(p);
					length += 1;
				} else if (partnerList.size() == NUM_OF_PARTNERS) {
					replacement(p);
				} else {
					System.out.println("error!!! -- neighborList.size > length!!!");
					System.exit(1);
				}
			}
		}
		
		public int indexOf(Partner p) {
			for(int i=0; i<partnerList.size(); i++) {
				if(partnerList.get(i).getPartnerId().compareTo(p.getPartnerId()) == 0) {
					return i;
				}
			}
			return -1;
		}
		
		public void increaseHealth(String uid) {
			Iterator<Partner> iterator = partnerList.iterator();
			while(iterator.hasNext()) {
				Partner tp = iterator.next();
				if(tp.getPartnerId().compareTo(uid) == 0) {
					tp.increaseHealth();
				}
			}
		}
		
		public void decreaseHealth(String uid) {
			Iterator<Partner> iterator = partnerList.iterator();
			while(iterator.hasNext()) {
				Partner tp = iterator.next();
				if(tp.getPartnerId().compareTo(uid) == 0) {
					tp.decreaseHealth();
				}
			}
		}
		
		public boolean contains(Partner p) {
			Iterator<Partner> iterator = partnerList.iterator();
			while(iterator.hasNext()) {
				if(iterator.next().getPartnerId().compareTo(p.getPartnerId()) == 0) {
					return true;
				}
			}
			return false;
		}
		
		public void refresh(Partner p) {
			Iterator<Partner> iterator = partnerList.iterator();
			while(iterator.hasNext()) {
				Partner tp = iterator.next();
				if(tp.getPartnerId().compareTo(p.getPartnerId()) == 0) {
					tp.refresh(p);
				}
			}
		}
		
		public void refresh(Partner p, int index) {
			Partner tp = partnerList.get(index);
			tp.refresh(p);
		}
		
		public void printQueue() {
			Iterator<Partner> iterator = partnerList.iterator();
			while(iterator.hasNext()) {
				Partner p = iterator.next();
				System.out.print(p.getPartnerId() + "(h:" + p.getHealthIndex() + ", s:" + p.getSimilarity() + ") ");
			}
			System.out.println();
		}
		
		//Most important, should consider the health and the similarity
		public void replacement(Partner p) {
			//Iterator<Partner> iterator = partnerList.iterator();
			/*while(iterator.hasNext()) {
				Partner tp = iterator.next();
				if(p.getHealthIndex() > tp.getHealthIndex()) {
					deletePartner(tp.getPartnerId());
					addPartner(p);
				} else if(p.getHealthIndex() == tp.getHealthIndex()) {
					if(p.getSimilarity() >= tp.getSimilarity()) {
						deletePartner(tp.getPartnerId());
						addPartner(p);
					}
				}
			}*/
			for(int i=0; i<partnerList.size(); i++) {
				Partner tp = partnerList.get(i);
				if(p.getHealthIndex() > tp.getHealthIndex()) {
					deletePartner(i);
					addPartner(p);
				} else if(p.getHealthIndex() == tp.getHealthIndex()) {
					if(p.getSimilarity() >= tp.getSimilarity()) {
						deletePartner(i);
						addPartner(p);
					}
				}
			}
			
			//partnerMap.writeBack();
		}
		
		public void deletePartner(int index) {
			partnerList.remove(index);
			length -= 1;
		}
		
		public void deletePartner(String pid) {
			for(int i=0; i<partnerList.size(); i++) {
				if(partnerList.get(i).getPartnerId().compareTo(pid) == 0) {
					partnerList.remove(i);
					length -= 1;
					return;
				}
			}
		}
		
		public String partners2string() {
			String result = "";
			Iterator<Partner> iterator = partnerList.iterator();
			while(iterator.hasNext()) {
				result = result + iterator.next().getPartnerId() + "#";
			}
			return result;
		}
		
		public String healthes2string() {
			String result = "";
			Iterator<Partner> iterator = partnerList.iterator();
			while(iterator.hasNext()) {
				result = result + iterator.next().getHealthIndex() + "#";
			}
			return result;
		}
		
		public String sims2string() {
			String result = "";
			Iterator<Partner> iterator = partnerList.iterator();
			while(iterator.hasNext()) {
				result = result + iterator.next().getSimilarity() + "#";
			}
			return result;
		}
		
		public int getLength() {
			return length;
		}
		
		public String getFolloweeId() {
			return followeeId;
		}
		
		public List<Partner> getPartnerList() {
			return partnerList;
		}
		
		public List<String> getPartnerIds() {
			ArrayList<String> al = new ArrayList<String>();
			Iterator<Partner> iterator = partnerList.iterator();
			while(iterator.hasNext()) {
				al.add(iterator.next().getPartnerId());
			}
			return al;
		}
	}
}